def test_yapf_success(testdir):
    testdir.makepyfile('''
        SOME_VALUE = 8
    ''')

    result = testdir.runpytest('--yapf', '-v')
    print(result)

    assert result.ret == 0


def test_yapf_failure(testdir):
    testdir.makepyfile('''
        AAA =8
    ''')

    result = testdir.runpytest('--yapf', '-v')

    result.stdout.fnmatch_lines([
        '*diff: +1/-1 lines',
    ])
    assert result.ret != 0


def test_yapf_failure_diff(testdir):
    testdir.makepyfile('''
        AAA =8
    ''')

    result = testdir.runpytest('--yapf', '--yapfdiff', '-v')

    result.stdout.fnmatch_lines(['*-AAA =8*', '*+AAA = 8*'])
    assert result.ret != 0


def test_yapf_error(testdir):
    testdir.makepyfile('''
        AAA
    ''')

    result = testdir.runpytest('--yapf', '-v')

    result.stdout.fnmatch_lines([
        'E   NameError: name \'AAA\' is not defined',
    ])
    assert result.ret != 0



"""
Validate your Python file format with yapf.
"""
import re

import pytest
from yapf.yapflib import file_resources
from yapf.yapflib.yapf_api import FormatFile


def pytest_addoption(parser):
    """
    Parse optional arguments configured as `addopts` in pytest.ini,
    or trasferred from CLI.
    """
    group = parser.getgroup('yapf')
    group.addoption(
        '--yapf',
        action='store_true',
        help='run yapf on *.py files.',
    )
    group.addoption(
        '--yapfdiff',
        action='store_true',
        help='show diff of yapf output.',
    )
    group.addoption(
        '--yapfstyle',
        action='store',
        dest='yapfstyle',
        default=None,
        help='style to be used by yapf.',
    )


def pytest_collect_file(path, parent):
    """
    Return collection Node or None for the given path.
    Any new node needs to have the specified parent as a parent.
    """
    config = parent.config
    if config.option.yapf and path.ext == '.py':
        return YapfItem(path, parent)
    return None


class YapfError(Exception):
    """
    Raise this with message when any yapf error occurs.
    """


class YapfItem(pytest.Item, pytest.File):
    """
    Run yapf for every file.
    """
    def __init__(self, path, parent):
        super(YapfItem, self).__init__(path, parent)
        self.path = str(path)
        self.show_diff = self.parent.config.option.yapfdiff is True
        self.style = (
            self.parent.config.getoption('yapfstyle') or
            file_resources.GetDefaultStyleForDir(self.path)
        )  # yapf: disable

    def runtest(self):
        """
        Run yapf with each file, raise YapfError with messages if failed.
        """
        diff, _, changed = FormatFile(
            self.path,
            style_config=self.style,
            print_diff=True,
        )
        if not changed:
            return

        if self.show_diff:
            message = diff
        else:
            added = sum(1 for i in re.finditer(r'^\+', diff, re.MULTILINE))
            removed = sum(1 for i in re.finditer(r'^\-', diff, re.MULTILINE))
            message = 'ERROR: {} YAPF diff: +{}/-{} lines'.format(
                self.path,
                added - 1,
                removed - 1,
            )
        raise YapfError(message)

    def repr_failure(self, excinfo, style=None):
        """
        Repare the failure YapfError.
        """
        if excinfo.errisinstance(YapfError):
            return excinfo.value.args[0]
        return super().repr_failure(excinfo, style)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Setup this package.
"""

from setuptools import find_packages, setup

_TEST_REQUIRES = [
    'mock',
    'pytest',
    'pytest-cov',
    'pytest-flake8',
    'pytest-isort',
    'pytest-mock',
    'pytest-pep8',
    'pytest-pylint',
]

setup(
    use_scm_version=True,
    zip_safe=False,
    # Package modules and data
    packages=find_packages('src'),
    package_dir={'': 'src'},
    # Entries
    entry_points={
        'pytest11': ('yapf = pytest_yapf3', ),
    },
    # Requires
    python_requires='>=3.4',
    install_requires=(
        'pytest>=3.1.1',
        'yapf>=0.16.2',
    ),
    setup_requires=[
        'pytest-runner',
        'setuptools-scm',
    ],
    tests_require=_TEST_REQUIRES,
    extras_require={
        'dev': _TEST_REQUIRES + [
            'flake8',
            'isort',
            'pylint',
        ],
    },
    # PyPI Metadata
    keywords=['pytest', 'yapf', 'sc'],
    platforms=['any'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Pytest',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Testing',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: MIT License',
    ],
)
